#ifndef GMGUIMGR_H
#define GMGUIMGR_H

#include "gmDefs.h"

#include <string>
#include <map>
#include <vector>
#include <cstring>

#include <sqlite3.h>

#include <wx/string.h>
#include <wx/thread.h>
#include <wx/filefn.h>

#include "AddrData.h"
#include "IO.ListMaker.BoardListMaker.h"
#include "IO.ListMaker.ThreadListMaker.h"
#include "gmModel.h"
#include "GikoMonaMain.h"
#include "TVWindow.h"
#include "GikoMonaApp.h"
#include "BbsMgr.h"

#include "Loki/AssocVector.h"

class GikoMonaFrame;
class TVWindow;

class gmGuiMgr {
    friend class GikoMonaApp;
    private:
        gmGuiMgr();
        virtual ~gmGuiMgr();

        struct BbsInfo {
            unsigned long parentId;
            wxString Key;
            BbsInfo(unsigned long id, const wxString& key) : parentId(id), Key(key) {}
            BbsInfo() {}
            ~BbsInfo() {}
        };

        sqlite3 *db;
        GikoMonaFrame *mainWnd;
        TVWindow *tvWnd;
        bool isOpenMainWindow, isOpenTVWindow;
        wxCriticalSection critSect;
        wxMutex *hostIdMutex;
        wxMutex *boardIdMutex;
        wxMutex *threadIdMutex;
        // 名前だと重複する可能性があるので、自前でIdを割り振ってやる。
        unsigned long hostId;
        unsigned long boardId;
        unsigned long threadId;
        Loki::AssocVector<unsigned long, BbsInfo> hostIdTable;
        Loki::AssocVector<unsigned long, BbsInfo> boardIdTable;
        Loki::AssocVector<unsigned long, BbsInfo> threadIdTable;

        template<typename T>
        class gmThread : public wxThread {
        public:
            /// @warning コンストラクタ中で処理を行うことは推奨されていない。
            gmThread(BBS::BbsMgr *bbsMgr, const BBS::AddressData& bbsInfo, T& ref_result)
                      : wxThread(wxTHREAD_JOINABLE ), mgr(bbsMgr), info(bbsInfo), result(ref_result) {}
            virtual ~gmThread() {delete mgr;}
            virtual void *Entry() = 0;
            const BBS::BbsMgr *getBbsMgr() const { return mgr; }
        protected:
            BBS::BbsMgr *mgr;
            const BBS::AddressData info;
            T& result;
        };

/*
        class PostString2Bbs : public gmThread<int> {
            PostString2BbsThread(BBS::BbsMgr *bbsMgr, const BBS::DataInfo& bbsInfo,
                                 int& result, const wxString& str) : wxThread(bbsMgr, bbsInfo, result) {
                postedString = str;
            }
            wxString postedString;
            void *Entry() {}
        };

        class CreateThreadInBbs : public gmThread<int> {
            CreateThreadInBbs(BBS::BbsMgr *bbsMgr, const BBS::DataInfo& bbsInfo,
                              int& result, const wxString& str) : gmThread(bbsMgr, bbsInfo, result) {
                postedString = str;
            }
            wxString postedString;
            void *Entry() {}
        };
*/

        class GetThreadFromBbs : public gmThread< std::pair<std::wstring, std::wstring> > {
        public:
            /// @warning コンストラクタ中で処理を行うことは推奨されていない。
            GetThreadFromBbs(BBS::BbsMgr *bbsMgr, const BBS::AddressData& bbsInfo,
                             std::pair<std::wstring, std::wstring>& ref_result, bool isLocal)
                              : gmThread< std::pair<std::wstring, std::wstring> >(bbsMgr, bbsInfo, ref_result) {
                                  isFileLocal = isLocal;
                              }
            void *Entry() {
                result = mgr->getThread(info, isFileLocal);
            }
        private:
            bool isFileLocal;
        };

        class GetThreadListFromBbs : public gmThread< std::vector<BBS::ListMaker::ThreadInfo> > {
        public:
            /// @warning コンストラクタ中で処理を行うことは推奨されていない。
            GetThreadListFromBbs(BBS::BbsMgr *bbsMgr, const BBS::AddressData& bbsInfo,
                                 std::vector<BBS::ListMaker::ThreadInfo>& ref_result, bool isLocal)
                                  : gmThread< std::vector<BBS::ListMaker::ThreadInfo> >(bbsMgr, bbsInfo, ref_result) {
                                     isFileLocal = isLocal;
                                 }
            void *Entry() {
                result = mgr->getTList(info, isFileLocal);
            }
        private:
            bool isFileLocal;
        };

        class GetBoardListFromBbs : public gmThread< std::vector< std::vector<BBS::ListMaker::BoardInfo> > > {
        public:
            /// @warning コンストラクタ中で処理を行うことは推奨されていない。
            GetBoardListFromBbs(BBS::BbsMgr *bbsMgr, const BBS::AddressData& bbsInfo,
                                std::vector< std::vector<BBS::ListMaker::BoardInfo> >& ref_result, bool isLocal)
                                 : gmThread< std::vector< std::vector<BBS::ListMaker::BoardInfo> > >(bbsMgr, bbsInfo, ref_result) {
                                    isFileLocal = isLocal;
                                }
            void *Entry() {
                result = mgr->getBoardList(info, isFileLocal);
            }
        private:
            bool isFileLocal;
        };


    protected:
    public:
        GikoMonaFrame *getMainWindow() const {
            return mainWnd;
        }

        TVWindow *getTVWindow() const {
            return tvWnd;
        }

        void showMainWindow();
        void showTVWindow();

        void createMainWindow(wxWindow* parent = NULL, wxWindowID id = -1);
        void createTVWindow(wxWindow* parent = NULL, wxWindowID id = -1);

        bool closeMainWindow();
        bool closeTVWindow();

        void getThreadFromBbs(std::pair<std::wstring, std::wstring>& data, unsigned long thrdId, const wxString& threadTitle, bool isLocal);
        void getTListFromBbs(std::vector<BBS::ListMaker::ThreadInfo>& v, unsigned long brdId, bool isLocal);
        void getBoardListFromBbs(std::vector< std::vector<BBS::ListMaker::BoardInfo> >& v, bool isLocal);
        void postResponse(wxEvent& event);
        void makeThread(wxEvent& event);
        const ResData getThread(unsigned long thrdId, const wxString& threadTitle, bool isLocal);
        const gmModel::ThreadInfoList getTList(unsigned long brdId, bool isLocal);
        const gmModel::BoardInfoLists getBoardList(bool isLocal);
        const ResData renewalThread(unsigned long thrdId, const wxString& threadTitle);
        const gmModel::ThreadInfoList renewalTList(unsigned long brdId);
        const gmModel::BoardInfoLists renewalBoardList();
};

#endif // GMGUIMGR_H
