#ifndef BBS_TCP_IP_CONNECTIONMGR_H
#define BBS_TCP_IP_CONNECTIONMGR_H

#include "lib2ch.h"
#include <map>
#include <exception>

#include <wx/protocol/http.h>
#include <wx/msgdlg.h>
#include "AddrData.h"

namespace BBS {
namespace TCP_IP {

class ResponseHeader;
struct ContentsInfo {
    ContentsInfo(wxString lstMod, const wxString& et, int state) : lastModefied(lstMod), eTag(et), isSuccess(state) {}
    ~ContentsInfo() {}
    wxString lastModefied;
    wxString eTag;
    int isSuccess;
};

class HostChangeException : public std::exception {
        wxString oldHostName, newHostName;
    public:
        HostChangeException(wxString oldHostName_, wxString newHostName_) : std::exception() {
            oldHostName = oldHostName_, newHostName = newHostName_;
        }
        ~HostChangeException() throw() {}
        wxString getNewHostName() const { return newHostName; }
        wxString getOldHostName() const { return oldHostName; }
        const char *what() const throw() {
            wxString msg = _T("鯖が移転しています\n")
                           _T("前鯖名：") + oldHostName + _T("\n")
                           _T("新鯖名：") + newHostName;
            wxMessageBox(msg, _T("ChangeHostException"));
        }
};

class BeLoginException : public std::exception {};
class MaruLoginException : public std::exception {};
class RegulationException : public std::exception {};
class Samba24Exception : public RegulationException {};

class ConnectionMgr {
    public:
        AddressData URLData;
        ConnectionMgr(const AddressData& temp) {
            URLData = temp;
        }
        ConnectionMgr(const AddressData& temp, const wxString& usrAgent) {
            URLData = temp;
            userAgent = usrAgent;
        }
        virtual ~ConnectionMgr() {}
        virtual ContentsInfo getBoardList() = 0;
        virtual ContentsInfo getThreadList() = 0;
        virtual ContentsInfo getThread() = 0;
        wxHTTP *Http;
    protected:
        virtual wxInputStream *connect(const wxString& path) = 0;
        ResponseHeader *header;
        wxString userAgent;
    private:
};

} // namespace TCP_IP
} // namespace BBS

#endif // BBS_TCP_IP_CONNECTIONMGR_H
