/**
*   sqlitepp
*/
#ifndef SQLITEPP_EXCEPTION_H
#define SQLITEPP_EXCEPTION_H

#include <exception>
#include <string>

namespace SQLite {
    class SQLiteException : public std::exception {
    protected:
        std::string 	what_;

    public:
        SQLiteException() : std::exception() {}

        SQLiteException(const char* what) : std::exception(), what_(what) {}

        virtual ~SQLiteException() throw() {}

        virtual const char* what() const throw() {
            return what_.c_str();
        }

        void setWhat(const char* what) throw() {
            what_ = what;
        }
    };


    class ConnectionException : public SQLiteException {
    public:
        ConnectionException() : SQLiteException() {}

        ConnectionException(const char* what) : SQLiteException(what) {
            setWhat(what);
        }

        virtual ~ConnectionException() throw() {}
    };


    class QueryException : public SQLiteException {
        /// SQLitẽG[R[h
        int		errorCode_;

    public:
        QueryException() : SQLiteException() {
            errorCode_ = SQLITE_ERROR;
        }

        QueryException(const char* what, int errCode)
         : SQLiteException(what), errorCode_(errCode) {
            setWhat(what);
        }

        /// O̎ʂ\ASQLitẽG[R[h擾B
        /**
         * @note SQLitẽG[R[h́Asqlite3.hɋLڂĂB<br>
         * iSQLITE_OK  SQLITE_DONE ܂ł̒萔j
         * @return G[R[h
         */
        int errorCode() {
            return errorCode_;
        }

        /// SQLitẽG[R[hZbg
        /**
         * @param errorCode G[R[h
         */
        void setErrorCode(int errorCode) {
            errorCode_ = errorCode;
        }
    };
}


#endif

