#include "IO.ListMaker.BoardListMaker.h"

#include <wx/textfile.h>
#include <wx/regex.h>
#include <wx/msgdlg.h>
#include <wx/xml/xml.h>
#include <wx/variant.h>
#include <wx/filefn.h>
#include <wx/wfstream.h>
#include <wx/uri.h>

#include <vector>
#include <algorithm>
#include <cstring>

using namespace std;

namespace BBS{
    namespace ListMaker{
        void BoardListMaker::makeList(const wxString& filename){
            BeginBoardURL = false;
            EndBoardURL = false;
            wxTextFile ifs(_T("BoardList.htm"));
            ifs.Open(wxConvLocal);
            wxString str;
            std::vector<BoardInfo> vBaseList;
            wxRegEx reCategory(_T("<B>([^>]+)</B>"));
            wxRegEx reBoardURL(_T("http://([^>]+)/"));
            wxRegEx reBoardName(_T(">([^>]+)</A>"));
            std::vector<wxXmlNode*> nodeList;
            if(wxFileExists(filename)) {
                //wxGetApp().gmLog.dbgWrite(_T("@BoardlistMaker::makeList ") _T(TO_STR(__LINE__)) _T(" : wxFileExists() == true."));
                if(!wxRemoveFile(filename)) {
                    //gmShowErrorDialog(_T("can't open / find / delete file."), _T("you have not report this log..."), _T("BoardListMaker"), _T("makeList"), _T(TO_STR(__LINE__)));
                }
            }
            wxFileOutputStream ofs(_T("2ch.brd"));
            wxString xmlstr = _T("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
            xmlstr += _T("<BoardList>\n");

            long boardId = 0;
            bool isFirst = true;
            for(str = ifs.GetFirstLine(); !ifs.Eof(); str = ifs.GetNextLine()) {
                if(str.find(_T("<A HREF=http://info.2ch.net/guide/>2ch総合案内</A>")) != wxString::npos){
                    BeginBoardURL = true;
                    continue;
                }else if(str.find(_T("【<B>チャット</B>】<A HREF=http://irc.2ch.net/ TARGET=_blank>２ｃｈ＠ＩＲＣ</A>")) != wxString::npos){
                    EndBoardURL = true;
                    break;
                }
                if(BeginBoardURL && (!EndBoardURL)){
                    if(str.find(_T("<B>")) != wxString::npos){
                        if(!reCategory.Matches(str)){
                            wxMessageBox(str, _T("エラー1"));
                        }
                        wxString brdName = reCategory.GetMatch(str);
                        brdName.Replace(_T("<B>"), _T(""));
                        brdName.Replace(_T("</B>"), _T(""));
                        if(isFirst) {
                            isFirst = false;
                        }else{
                            xmlstr += _T("    </category>\n");
                        }
                        xmlstr += _T("    <category name=\"");
                        xmlstr += brdName + _T("\">\n");
                    }
                    if(!reBoardURL.Matches(str)){
                        wxMessageBox(str, _T("エラー2"));
                    }
                    if(!reBoardName.Matches(str)){
                        wxMessageBox(str, _T("エラー3"));
                    }
                    wxString url = reBoardURL.GetMatch(str);
                    wxString brdName = reBoardName.GetMatch(str);
                    brdName.Replace(_T(">"), _T(""));
                    brdName.Replace(_T("</A"), _T(""));
                    if(brdName.find(_T("&")) != wxString::npos) {
                        brdName.Replace(_T("&"), _T("&amp;"));
                    }
                    wxString idStr;
                    wxVariant val(boardId);
                    val.Convert(&idStr);
                    xmlstr += _T("        <board ");
                    xmlstr += _T("Url=\"") + url + _T("\" ");
                    xmlstr += _T("Name=\"") + brdName + _T("\" ");
                    xmlstr += _T("Id=\"") + idStr + _T("\"/>\n");
                    boardId++;
                }
            }
            xmlstr += _T("    </category>");
            xmlstr += _T("</BoardList>");
            const wxWX2MBbuf buf(xmlstr.mb_str(wxConvUTF8));
            if(!buf) {
                return;
            }
            ofs.Write((const char*)buf, strlen((const char*)buf));
            ifs.Close();
            ofs.Close();
        }

        const vector< vector<BoardInfo> > BoardListMaker::read(const wxString& fileName) {
            if(wxFileExists(fileName)) {
                wxXmlDocument doc;
                if(doc.Load(fileName)) {
                    //wxGetApp().gmLog.dbgWrite(_T("@BoardlistMaker::makeList ") _T(TO_STR(__LINE__)) _T(" : wxXmlNode::Save() == true."));
                }else{
                    /*wxGetApp().gmLog
                    #ifdef __WXDEBUG__
                        .dbgWrite(
                    #else
                        .write(
                    #endif
                               _T("@BoardlistMaker::makeList") _T(TO_STR(__LINE__)) _T(" : \"2ch.brd\"が不正です。。。"));
                    */return vList;
                }
                wxXmlNode *catNode = doc.GetRoot()->GetChildren();
                vector<BoardInfo> vBaseList;
                while(catNode) {
                    BoardInfo catInfo;
                    catInfo.BoardName = catNode->GetProperties()->GetValue();
                    vBaseList.push_back(catInfo);
                    wxXmlNode *brdNode = catNode->GetChildren();
                    while(brdNode) {
                        wxXmlProperty *brdProp = brdNode->GetProperties();
                        BoardInfo info;
                        for(int i = 0; i < 3; i++) {
                            switch (i) {
                                case 0:
                                    wxString url = brdProp->GetValue();
                                    wxURI uri(url);
                                    wxString hostKey = uri.GetServer(); // HostKey
                                    wxString path = uri.GetPath();
                                    size_t first = path.find(_T("/")) + 1;
                                    wxString boardKey = path.Mid(first, path.find(_T("/"), first)); // BoardKey
                                    boardKey.Replace(_T("/"), _T(""));
                                    info.URL.HostKey = hostKey;
                                    info.URL.BoardKey = boardKey;
                                    break;

                                case 1:
                                    info.BoardName = brdProp->GetValue();
                                    break;

                                case 2:
                                    info.BoardId = brdProp->GetValue();
                                    break;
                            }
                            brdProp = brdProp->GetNext();
                        }
                        vBaseList.push_back(info);
                        brdNode = brdNode->GetNext();
                    }
                    vList.push_back(vBaseList);
                    catNode = catNode->GetNext();
                    vBaseList.clear();
                }
            }
            return vList;
        }

        wxString BoardListMaker::searchBoardNameFromBoardKey(const wxString& fileName, const wxString& boardKey) {
            vector< vector<BoardInfo> > v = read(fileName);
            vector< vector<BoardInfo> >::iterator catItr = v.begin();
            while(catItr != v.end()) {
                vector<BoardInfo>::iterator brdItr = catItr->begin();
                while(brdItr != catItr->end()) {
                    if(brdItr->URL.find(boardKey) != wxString::npos) {
                        return brdItr->BoardName;
                    }
                    brdItr++;
                }
                catItr++;
            }
            return wxEmptyString;
        }

        bool BoardListMaker::changeHostName(const wxString& fileName, const wxString& oldHostName, const wxString& newHostName) {
            if(wxFileExists(fileName)) {
                wxTextFile ifs(fileName);
                if(!ifs.IsOpened()) { return false; }
                wxString str;
                bool isChangedHost;
                for(isChangedHost = false, str = ifs.GetFirstLine();
                        !ifs.Eof();
                        str = ifs.GetNextLine()) {
                    if(str.find(oldHostName) != wxString::npos) {
                        isChangedHost = true;
                        str.Replace(oldHostName, newHostName, false);
                        continue;
                    } else if(isChangedHost) {
                        break;
                    }
                }
                if(!ifs.Close()) { return false; }
            } else {
                return false;
            }
            return true;
        }
    }
}

