#include <wx/zstream.h>
#include <wx/sstream.h>
#include <wx/wfstream.h>
#include <wx/file.h>
#include <wx/filefn.h>
#include <wx/filename.h>
#include <wx/regex.h>

#include "Connect2ch.h"
#include "Header4Channel2.h"

namespace BBS {
    namespace TCP_IP {

        Connect2ch::Connect2ch(const AddressData& temp) : ConnectionMgr::ConnectionMgr(temp){
            header = new Header4Channel2;
            Http = new wxHTTP;
        }

        Connect2ch::Connect2ch(const AddressData& temp, const wxString& usrAgent)
             : ConnectionMgr::ConnectionMgr(temp, usrAgent) {
            header = new Header4Channel2(usrAgent);
            Http = new wxHTTP;
        }

        Connect2ch::~Connect2ch() {
            Http->Close();
            Http->Destroy();
        }

        wxInputStream *Connect2ch::connect(const wxString& path) {
            Http->SetTimeout(30);
            Http->Connect(URLData.HostKey);
            wxInputStream *input = Http->GetInputStream(path);
            bool isEncodedGzip = false;
            if(Http->GetError() == wxPROTO_NOERR) {
                wxString responseHeader = Http->GetHeader(_T("Content-Encoding"));
                wxZlibInputStream *decompresser;
                if(responseHeader == _T("gzip")) {
                    decompresser = new wxZlibInputStream(*input, wxZLIB_GZIP);
                    isEncodedGzip = true;
                }
                if(Http->GetResponse() == 302) {
                    wxStringOutputStream *sstrm = new wxStringOutputStream;
                    if(isEncodedGzip) {
                        decompresser->Read(*sstrm);
                    }else {
                        input->Read(*sstrm);
                    }
                    wxString page = sstrm->GetString();
                    if(page.find(_T("<p>The document has moved")) != wxString::npos) {
                        wxRegEx reg(_T("<a href=\"http://.*/.*/\">"));
                        if(reg.GetMatchCount()) {
                            wxString newBrdUrl = reg.GetMatch(page);
                            wxString tmp = newBrdUrl.Mid(16);
                            wxString newHostName = tmp.Mid(0, tmp.find(_T("/")));
                            if(isEncodedGzip) {
                                delete decompresser;
                            }else {
                                delete input;
                            }
                            throw HostChangeException(URLData.HostKey, newHostName);
                        } else {
                            return NULL;
                        }
                    }
                }
                if(isEncodedGzip) {
                    return decompresser;
                }else {
                    return input;
                }
            } else {
                // 板が移転している可能性がある
                if(Http->GetResponse() == 404) {
                    Http->Reconnect();
                    wxString idxPagePath = _T("/") + URLData.BoardKey + _T("/index.html");
                    wxInputStream *idxPageStrm = Http->GetInputStream(idxPagePath);
                    if(Http->GetError() == wxPROTO_NOERR) {
                        wxString responseHeader = Http->GetHeader(_T("Content-Encoding"));
                        wxZlibInputStream *decompresser;
                        if(responseHeader == _T("gzip")) {
                            decompresser = new wxZlibInputStream(*idxPageStrm, wxZLIB_GZIP);
                            delete idxPageStrm;
                            idxPageStrm = decompresser;
                        }
                        wxStringOutputStream *sstrm = new wxStringOutputStream;
                        idxPageStrm->Read(*sstrm);
                        wxString idxPage = sstrm->GetString();
                        // 確実に移転してる
                        if(idxPage.find(_T("<title>2chbbs..</title>")) != wxString::npos) {
                            wxRegEx reg(_T("<a href=\"http://.*/.*/\">"));
                            if(reg.GetMatchCount()) {
                                wxString newBrdUrl = reg.GetMatch(idxPage);
                                wxString tmp = newBrdUrl.Mid(16);
                                wxString newHostName = tmp.Mid(0, tmp.find(_T("/")));
                                throw HostChangeException(URLData.HostKey, newHostName);
                            } else {
                                return NULL;
                            }
                        }
                    }
                }
                return NULL;
            }
        }

        ContentsInfo Connect2ch::getBoardList() {
            header->Header_getBoardList(this);
            wxInputStream *strm = connect(_T("/bbstable.html"));
            ContentsInfo inf(Http->GetHeader(_T("Last-Modified")), Http->GetHeader(_T("ETag")), Http->GetResponse());
            if(strm) {
                wxFileOutputStream Output(_T("BoardList.htm"));
                Output.Write(*strm);
                delete strm;
            }
            return inf;
        }

        ContentsInfo Connect2ch::getThreadList() {
            header->Header_getTList(this);
            wxString path = _T("/") + URLData.BoardKey + _T("/subject.txt");
            wxInputStream *strm = connect(path);
            ContentsInfo inf(Http->GetHeader(_T("Last-Modified")), Http->GetHeader(_T("ETag")), Http->GetResponse());
            if(strm) {
                wxString dirName = wxGetCwd();
                wxFileName dirMgr(dirName);
                dirName += _T("\\Log\\2ch.net\\") + URLData.BoardKey;
                dirMgr.Mkdir(dirName, 0777,  wxPATH_MKDIR_FULL);
                dirName += _T("\\subject.txt");
                wxFileOutputStream Output(dirName);
                Output.Write(*strm);
                delete strm;
            } else {
            }
            return inf;
        }

        ContentsInfo Connect2ch::getThread() {
            header->Header_getThread(this);
            wxString path = _T("/") + URLData.BoardKey + _T("/dat/") + URLData.ThreadKey + _T(".dat");
            wxInputStream *strm = connect(path);
            ContentsInfo inf(Http->GetHeader(_T("Last-Modified")), Http->GetHeader(_T("ETag")), Http->GetResponse());
            if(strm) {
                wxString dirName = wxGetCwd();
                wxFileName dirMgr(dirName);
                wxString tKey;
                tKey = URLData.ThreadKey.Mid(0, 4);
                dirName += _T("\\Log\\2ch.net\\") + URLData.BoardKey + _T("\\") + tKey;
                dirMgr.Mkdir(dirName, 0777,  wxPATH_MKDIR_FULL);
                dirName += _T("\\") + URLData.ThreadKey + _T(".dat");
                wxFileOutputStream Output(dirName);
                Output.Write(*strm);
                delete strm;
            }
            return inf;
        }

    } // namespace TCP_IP
} // namespace BBS
