#ifndef LIBTWITTER_HPP_INCLUDED
#define LIBTWITTER_HPP_INCLUDED

/*
*   Twitter for c++ Library
*   written by caprice <gikomona.2chbrowser@gmail.com>
*   created : 2010/06/20
*
*   thank you for using this library.
*   this library is to use twitter's api.
*   this library use these connection component :
*       boost::asio <http://boost.org>
*       wxHTTP      <http://wxwidgets.org/>
*       clx::http   <http://sourceforge.jp/projects/clxcpp/>
*   this library's license is BSD license <http://www.opensource.org/licenses/bsd-license.php>.
*/

/*
*   build options
*
*   WITH_WX     : when connect twitter, use wxHTTP. and base string use wxString.
*   WITH_BOOST  : when connect twitter, use boost::asio. and base string use std::tring.
*   WITH_CLX    : when connect twitter, use wxHTTP. and base string use std::string.
*/

#if     defined(WITH_BOOST)
    #include <boost/asio.hpp>
#elif   defined(WITH_CLX)
    #include <clx/http.h>
#elif   defined(WITH_WX)
    #include <wx/protocol/http.h>
#else
    #error build error.
#endif

#if defined(WITH_BOOST) && defined(WITH_CLX)
    /*
    * I except that "char" and "std::string" can hold UTF-8 and default charset is UTF-8.
    * if you use
    *   gcc : -fexec-charset=utf-8
    *   vc  :
    *   bcc :
    *   ow  :
    *   dmc :
    */
    typedef char Char;
    typedef std::string String;
    #define getUTF8(str) str; // for wx.
    #define Tx(s) s
#else
    /*
    * I except that "UNICODE" build option is enabled.
    * if you use wxWidgets in ansi-build, you mustn't use this library.
    */
    typedef Char wxChar;
    typedef String wxString;
    #ifndef UNICODE
        #error must enable "UNICODE" option !
    #endif
    #define NO_EXCEPTION
    #define getUTF8(str) str.ToUTF8(); // wxString's defalt charset is UTF-16 or 32 (I wrong?) so, I use this macro.
    #define Tx(s) wxT(s)
#endif

namespace oAuth {
    const static Char *consumerKey = Tx("");
    const static Char *consumerSecretKey = Tx("");
}


#endif // LIBTWITTER_HPP_INCLUDED

