#include "gmDefs.h"
#include "gmGuiMgr.h"
#include "Mgr2ch.h"
#include <wx/uri.h>
#include <wx/filefn.h>

using namespace std;

gmGuiMgr::gmGuiMgr() {
    hostIdMutex = new wxMutex;
    boardIdMutex = new wxMutex;
    threadIdMutex = new wxMutex;
    hostId = threadId = boardId = 0;
    tvWnd = NULL;
    mainWnd = NULL;
    isOpenMainWindow = false;
    isOpenTVWindow = false;
}

gmGuiMgr::~gmGuiMgr() {
    delete hostIdMutex;
    delete boardIdMutex;
    delete threadIdMutex;
}

void gmGuiMgr::showMainWindow() {
    if(isOpenMainWindow) {
        mainWnd->SetFocus();
        return;
    }
    if(wxFileExists(_T("2ch.brd"))) {
        mainWnd->getBoardList(true);
    }
    isOpenMainWindow = mainWnd->Show();
}

void gmGuiMgr::showTVWindow() {
    if(isOpenTVWindow) {
        tvWnd->SetFocus();
        return;
    }
    isOpenTVWindow = tvWnd->Show();
}

void gmGuiMgr::createMainWindow(wxWindow* parent, wxWindowID id) {
    if(!isOpenMainWindow) {
        mainWnd = new GikoMonaFrame(this, parent, id);
    }
}

void gmGuiMgr::createTVWindow(wxWindow* parent, wxWindowID id) {
    if(!isOpenTVWindow) {
        tvWnd = new TVWindow(this, parent, id);
    }
}

bool gmGuiMgr::closeMainWindow() {
    isOpenMainWindow = false;
    return mainWnd->Destroy();
}

bool gmGuiMgr::closeTVWindow() {
    isOpenTVWindow = false;
    return tvWnd->Destroy();
}


void gmGuiMgr::getBoardListFromBbs(vector< vector<BBS::ListMaker::BoardInfo> >& v, bool isLocal) {
    BBS::AddressData dataInfo(_T("menu.2ch.net"));
    GetBoardListFromBbs *thread = new GetBoardListFromBbs(new BBS::Mgr2ch(_T(gm_UA)), dataInfo, v, isLocal);
    thread->Create();
    thread->Run();
    thread->Wait();
    delete thread;
}

void gmGuiMgr::getTListFromBbs(vector<BBS::ListMaker::ThreadInfo>& v, unsigned long brdId, bool isLocal) {
    BbsInfo boardInfo = boardIdTable[brdId];
    BbsInfo hostInfo = hostIdTable[boardInfo.parentId];
    BBS::AddressData info(hostInfo.Key, boardInfo.Key);
    GetThreadListFromBbs *thread = new GetThreadListFromBbs(new BBS::Mgr2ch(_T(gm_UA)), info, v, isLocal);
    thread->Create();
    thread->Run();
    thread->Wait();
    delete thread;
}

void gmGuiMgr::getThreadFromBbs(pair<wstring, wstring>& data, unsigned long thrdId, const wxString& threadTitle, bool isLocal) {
    BbsInfo threadInfo = threadIdTable[thrdId];
    BbsInfo boardInfo = boardIdTable[threadInfo.parentId];
    BbsInfo hostInfo = hostIdTable[boardInfo.parentId];
    const wxString dirName = wxGetCwd() + _T("\\Log\\2ch.net\\") + boardInfo.Key + _T("\\subject.txt");
    BBS::ListMaker::ThreadListMaker TLMaker;
    wxString threadKey = TLMaker.readThreadURL(dirName, threadTitle);
    const BBS::AddressData info(hostInfo.Key, boardInfo.Key, threadKey);
    GetThreadFromBbs *thread = new GetThreadFromBbs(new BBS::Mgr2ch(_T(gm_UA)), info, data, isLocal);
    thread->Create();
    thread->Run();
    thread->Wait();
    delete thread;
}

const vector< vector<BoardInfo> > gmGuiMgr::getBoardList(bool isLocal) {
    vector< vector<BBS::ListMaker::BoardInfo> > v;
    vector< vector<BoardInfo> > boardList;
    vector<BoardInfo> boardListBase;
    getBoardListFromBbs(v, isLocal);
    hostIdMutex->Lock();
    boardIdMutex->Lock();
    for(vector< vector<BBS::ListMaker::BoardInfo> >::iterator itr = v.begin(); itr != v.end(); itr++){
        for(vector<BBS::ListMaker::BoardInfo>::iterator baseItr = itr->begin(); baseItr != itr->end(); baseItr++){
            hostId++;
            boardId++;
            BBS::AddressData url = baseItr->URL;
            BbsInfo boardInfo(hostId, url.BoardKey);
            BbsInfo hostInfo(0, url.HostKey);
            boardIdTable[boardId] = boardInfo;
            hostIdTable[hostId] = hostInfo;
            BoardInfo info(baseItr->BoardName, boardId);
            boardListBase.push_back(info);
        }
        boardList.push_back(boardListBase);
        boardListBase.clear();
    }
    boardIdMutex->Unlock();
    hostIdMutex->Unlock();
    return boardList;
}

const vector<ThreadInfo> gmGuiMgr::getTList(unsigned long brdId, bool isLocal) {
    vector<BBS::ListMaker::ThreadInfo> v;
    vector<ThreadInfo> threadList;
    getTListFromBbs(v, brdId, isLocal);
    threadIdMutex->Lock();
    for(vector<BBS::ListMaker::ThreadInfo>::iterator itr = v.begin(); itr != v.end(); itr++){
        threadId++;
        ThreadInfo threadInfo(itr->Title, itr->ResCount, threadId);
        threadList.push_back(threadInfo);
        BbsInfo bbsInfo(brdId, wxEmptyString);
        threadIdTable[threadId] = bbsInfo;
    }
    threadIdMutex->Unlock();
    return threadList;
}

const ResData gmGuiMgr::getThread(unsigned long thrdId, const wxString& threadTitle, bool isLocal) {
    pair<wstring, wstring> data;
    getThreadFromBbs(data, thrdId, threadTitle, isLocal);
    const ResData rdata(data.first.c_str(), data.second.c_str(), thrdId);
    return rdata;
}

void gmGuiMgr::postResponse(wxEvent& event) {

}

void gmGuiMgr::makeThread(wxEvent& event) {

}

const ResData gmGuiMgr::renewalThread(unsigned long thrdId, const wxString& threadTitle) {
    return getThread(thrdId, threadTitle, false);
}

const vector<ThreadInfo> gmGuiMgr::renewalTList(unsigned long brdId) {
/// TODO (caprice#1#): この下はまた今度。
}

const vector< vector<BoardInfo> > gmGuiMgr::renewalBoardList() {
/// TODO (caprice#1#): この下はまた今度。
}

